// Enforce use of a strict operating context.
'use strict';

// Define handler for this plugin.
const plugin = (nodewrite, options, nextPlugin) => {

  // Use provided plugin options.
  const { container } = options;

  // Register additional npm dependencies required by this plugin.
  container.register('camelCase', () => require('lodash/camelCase'));
  container.register('upperFirst', () => require('lodash/upperFirst'));
  container.register('dotProp', () => require('dot-prop'));
  container.register('JSData', () => require('js-data'));
  container.register('JSDataRethinkDbAdapter', () => require('js-data-rethinkdb'));
  container.register('rethinkdbAdapter', (config, JSDataRethinkDbAdapter) => {

    const rethinkdbAdapter = new JSDataRethinkDbAdapter({
      host: config.Core.Data.rethinkdb.host,
      port: config.Core.Data.rethinkdb.port,
      db: config.Core.Data.rethinkdb.database,
      user: config.Core.Data.rethinkdb.username,
      password: config.Core.Data.rethinkdb.password,
      servers: [
        {
          host: config.Core.Data.rethinkdb.host,
          port: config.Core.Data.rethinkdb.port
        }
      ],
      discovery: true,
      pool: true,
      buffer: 5,
      max: 1000,
      timeout: 20,
      pingInterval: -1,
      timeoutError: 1000,
      timeoutGb: 60 * 60 * 1000,
      maxExponent: 6,
      silent: true
    });
    return rethinkdbAdapter;
  });
  container.register('store', (rethinkdbAdapter, JSData) => {

    const store = new JSData.DS({
      debug: false,
      bypassCache: true,
      cacheResponse: false,
      clearEmptyQueries: true,
      keepChangeHistory: false,
      log: false,
      notify: false,
      resetHistoryOnInject: true,
      upsert: false,
      useFilter: true
    });
    store.registerAdapter('rethinkdb', rethinkdbAdapter, { default: true });
    return store;
  });

  // Resolve dependencies from app container.
  container.resolve((config, store, glob, path, camelCase, upperFirst, dotProp) => {

    // Extend server with a helper method for accessing core data store.
    nodewrite.decorate('server', 'store', () => store);

    // Extend server with a helper method for registering plugin models.
    nodewrite.decorate('server', 'registerPluginModels', (base, callback) => {
      // Set namespace.
      let namespace = base.replace(/.*?nodewrite-/, '');
      const name = path.parse(namespace).name;
      if (name.includes('core-') === true) {
        namespace = ['Core', upperFirst(camelCase(namespace.replace('core-', '')))];
      }
      if (name.includes('plugin-') === true) {
        namespace = ['Plugin', upperFirst(camelCase(namespace.replace('plugin-', '')))];
      }
      // Find nodewrite models installed under plugin base directory.
      glob('models/**/*.js', { cwd: base }, (error, models) => {

        if (error) {
          throw error;
        }
        // Iterate each model.
        for (let model of models) {
          // Build model parts object.
          model = {
            namespace: namespace.join('.'),
            name: path.parse(model).name,
            path: path.join(base, model)
          };
          const dotPath = [
            model.namespace,
            model.path.replace(`${base}/models/`, '').replace(`${model.name}.js`, '').split('/').map((path) => upperFirst(camelCase(path))).filter(Boolean).join('.'),
            model.name
          ].filter(Boolean).join('.');
          // Register model with data store.
          dotProp.set(store, dotPath, require(model.path)(store));
          // server.log(['nodewrite'], `Registered data model: ${dotPath}`);
        }

        if (callback) {
          callback();
        }
      });
    });

    // Extend server with a helper method for data pagination.
    nodewrite.decorate('server', 'paginate', (data) => {

      const totalItems = data.total;
      let itemsPerPage = data.limit;
      let currentPage = data.page;
      // Items per page must be at least one
      itemsPerPage = Math.max(1, itemsPerPage);
      // Calculate total pages
      const totalPages = Math.ceil(totalItems / itemsPerPage);
      // Current page must be at least one
      currentPage = Math.max(1, currentPage);
      // Determine previous/next pages
      const previousPage = currentPage > 1 ? currentPage - 1 : null;
      const nextPage = currentPage < totalPages ? currentPage + 1 : null;
      return {
        currentPage,
        itemsPerPage,
        nextPage,
        previousPage,
        totalItems,
        totalPages
      };
    });

    // Finished with plugin registration.
    nextPlugin();

  });

};

// Attach plugin attributes.
plugin.attributes = {
  pkg: require('./package.json')
};

// Expose plugin on module's register property.
exports.register = plugin;
